@@ -18,7 +18,17 @@
 # Copyright 2019 Wan Liuyang <tsfdye@gmail.com>                                #
 # Copyright 2020 Steve Kowalik <steven@wedontsleep.org>                        #
 # Copyright 2021 Steve Kowalik <steven@wedontsleep.org>                        #
-# Copyright 2023 Jonathan Leitschuh <Jonathan.Leitschuh@gmail.com>             #
+# Copyright 2023 Christoph Reiter <reiter.christoph@gmail.com>                 #
+# Copyright 2023 Enrico Minack <github@enrico.minack.dev>                      #
+# Copyright 2023 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
+# Copyright 2023 Jonathan Greg <31892308+jmgreg31@users.noreply.github.com>    #
+# Copyright 2023 Jonathan Leitschuh <jonathan.leitschuh@gmail.com>             #
+# Copyright 2023 Joseph Henrich <crimsonknave@gmail.com>                       #
+# Copyright 2023 Nicolas Schweitzer <nicolas.schweitzer@datadoghq.com>         #
+# Copyright 2023 Trim21 <trim21.me@gmail.com>                                  #
+# Copyright 2024 Enrico Minack <github@enrico.minack.dev>                      #
+# Copyright 2024 Jirka Borovec <6035284+Borda@users.noreply.github.com>        #
+# Copyright 2024 Min RK <benjaminrk@gmail.com>                                 #
 #                                                                              #
 # This file is part of PyGithub.                                               #
 # http://pygithub.readthedocs.io/                                              #
@@ -38,12 +48,14 @@
 #                                                                              #
 ################################################################################
 
+import email.utils
+import re
 import typing
 from datetime import datetime, timezone
+from decimal import Decimal
 from operator import itemgetter
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Type, Union, overload
 
-from dateutil import parser
 from typing_extensions import Protocol, TypeGuard
 
 from . import Consts
@@ -64,6 +76,37 @@ class Attribute(Protocol[T_co]):
         raise NotImplementedError
 
 
+def _datetime_from_http_date(value: str) -> datetime:
+    """
+    Convert an HTTP date to a datetime object.
+
+    Raises ValueError for invalid dates.
+
+    """
+
+    dt = email.utils.parsedate_to_datetime(value)
+    if dt.tzinfo is None:
+        # RFC7231 states that UTC is assumed if no timezone info is present
+        return dt.replace(tzinfo=timezone.utc)
+    return dt
+
+
+def _datetime_from_github_isoformat(value: str) -> datetime:
+    """
+    Convert an GitHub API timestamps to a datetime object.
+
+    Raises ValueError for invalid timestamps.
+
+    """
+
+    # Github always returns YYYY-MM-DDTHH:MM:SSZ, so we can use the stdlib parser
+    # with some minor adjustments for Python < 3.11 which doesn't support "Z"
+    # https://docs.github.com/en/rest/overview/resources-in-the-rest-api#schema
+    if value.endswith("Z"):
+        value = value[:-1] + "+00:00"
+    return datetime.fromisoformat(value)
+
+
 class _NotSetType:
     def __repr__(self) -> str:
         return "NotSet"
@@ -98,6 +141,62 @@ def is_optional_list(v: Any, type: Union[Type, Tuple[Type, ...]]) -> bool:
     return isinstance(v, _NotSetType) or isinstance(v, list) and all(isinstance(element, type) for element in v)
 
 
+camel_to_snake_case_regexp = re.compile(r"(?<!^)(?=[A-Z])")
+
+
+@overload
+def as_rest_api_attributes(graphql_attributes: Dict[str, Any]) -> Dict[str, Any]:
+    ...
+
+
+@overload
+def as_rest_api_attributes(graphql_attributes: None) -> None:
+    ...
+
+
+def as_rest_api_attributes(graphql_attributes: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:
+    """
+    Converts attributes from GraphQL schema to REST API schema.
+
+    The GraphQL API uses lower camel case (e.g. createdAt), whereas REST API uses snake case (created_at). Initializing
+    REST API GithubObjects from GraphQL API attributes requires transformation provided by this method.
+
+    Further renames GraphQL attributes to REST API attributes where the case conversion is not sufficient. For example,
+    GraphQL attribute 'id' is equivalent to REST API attribute 'node_id'.
+
+    """
+    if graphql_attributes is None:
+        return None
+
+    attribute_translation = {
+        "id": "node_id",
+        "databaseId": "id",  # must be after 'id': 'node_id'!
+        "url": "html_url",
+    }
+
+    def translate(attr: str) -> str:
+        def un_capitalize(match: re.Match) -> str:
+            return match.group(1) + match.group(2).lower()
+
+        attr = attribute_translation.get(attr, attr)
+        attr = re.sub(r"([A-Z])([A-Z]+)", un_capitalize, attr)
+        attr = camel_to_snake_case_regexp.sub("_", attr)
+        attr = attr.lower()
+
+        return attr
+
+    return {
+        translate(k): as_rest_api_attributes(v)
+        if isinstance(v, dict)
+        else (as_rest_api_attributes_list(v) if isinstance(v, list) else v)
+        for k, v in graphql_attributes.items()
+    }
+
+
+def as_rest_api_attributes_list(graphql_attributes: List[Optional[Dict[str, Any]]]) -> List[Optional[Dict[str, Any]]]:
+    return [as_rest_api_attributes(v) if isinstance(v, dict) else v for v in graphql_attributes]
+
+
 class _ValuedAttribute(Attribute[T]):
     def __init__(self, value: T):
         self._value = value
@@ -131,6 +230,14 @@ class GithubObject:
     CHECK_AFTER_INIT_FLAG = False
     _url: Attribute[str]
 
+    @classmethod
+    def is_rest(cls) -> bool:
+        return not cls.is_graphql()
+
+    @classmethod
+    def is_graphql(cls) -> bool:
+        return False
+
     @classmethod
     def setCheckAfterInitFlag(cls, flag: bool) -> None:
         cls.CHECK_AFTER_INIT_FLAG = flag
@@ -158,6 +265,16 @@ class GithubObject:
         self._rawData = attributes
         self._useAttributes(attributes)
 
+    @property
+    def requester(self) -> "Requester":
+        """
+        Return my Requester object.
+
+        For example, to make requests to API endpoints not yet supported by PyGitHub.
+
+        """
+        return self._requester
+
     @property
     def raw_data(self) -> Dict[str, Any]:
         """
@@ -212,6 +329,10 @@ class GithubObject:
     def _makeIntAttribute(value: Optional[Union[int, str]]) -> Attribute[int]:
         return GithubObject.__makeSimpleAttribute(value, int)
 
+    @staticmethod
+    def _makeDecimalAttribute(value: Optional[Decimal]) -> Attribute[Decimal]:
+        return GithubObject.__makeSimpleAttribute(value, Decimal)
+
     @staticmethod
     def _makeFloatAttribute(value: Optional[float]) -> Attribute[float]:
         return GithubObject.__makeSimpleAttribute(value, float)
@@ -234,7 +355,11 @@ class GithubObject:
 
     @staticmethod
     def _makeDatetimeAttribute(value: Optional[str]) -> Attribute[datetime]:
-        return GithubObject.__makeTransformedAttribute(value, str, parser.parse)  # type: ignore
+        return GithubObject.__makeTransformedAttribute(value, str, _datetime_from_github_isoformat)  # type: ignore
+
+    @staticmethod
+    def _makeHttpDatetimeAttribute(value: Optional[str]) -> Attribute[datetime]:
+        return GithubObject.__makeTransformedAttribute(value, str, _datetime_from_http_date)  # type: ignore
 
     def _makeClassAttribute(self, klass: Type[T_gh], value: Any) -> Attribute[T_gh]:
         return GithubObject.__makeTransformedAttribute(
@@ -302,6 +427,13 @@ class GithubObject:
         """
         return self._headers.get(Consts.RES_LAST_MODIFIED)  # type: ignore
 
+    @property
+    def last_modified_datetime(self) -> Optional[datetime]:
+        """
+        :type: datetime
+        """
+        return self._makeHttpDatetimeAttribute(self.last_modified).value  # type: ignore
+
     def get__repr__(self, params: Dict[str, Any]) -> str:
         """
         Converts the object to a nicely printable string.
@@ -331,6 +463,12 @@ class GithubObject:
         raise NotImplementedError("BUG: Not Implemented _completeIfNeeded")
 
 
+class GraphQlObject:
+    @classmethod
+    def is_graphql(cls) -> bool:
+        return True
+
+
 class NonCompletableGithubObject(GithubObject):
     def _completeIfNeeded(self) -> None:
         pass
@@ -373,8 +511,8 @@ class CompletableGithubObject(GithubObject):
 
     def update(self, additional_headers: Optional[Dict[str, Any]] = None) -> bool:
         """
-        Check and update the object with conditional request
-        :rtype: Boolean value indicating whether the object is changed
+        Check and update the object with conditional request :rtype: Boolean value indicating whether the object is
+        changed.
         """
         conditionalRequestHeader = dict()
         if self.etag is not None:
